@@ -127,7 +127,7 @@ Pauli Matrices API
    labels
 """
 
-from typing import Sequence, Optional, Union
+from typing import Sequence, Optional, Union, Tuple
 from types import ModuleType
 import string
 import numpy as np
@@ -139,7 +139,7 @@ except ImportError:
 else:
     has_jax = True
 
-from ._types import array_like, ndarray, MatrixDimension
+from ._types import ArrayType, array_like, MatrixDimension
 
 def paulis(dim: MatrixDimension) -> np.ndarray:
     r"""Return a list of generalized Pauli matrices of given dimension(s) as an array.
@@ -208,11 +208,26 @@ def paulis(dim: MatrixDimension) -> np.ndarray:
         return np.einsum(indices, *subsystems).reshape(*shape) / (2 ** (num_sub - 1))
 
 
+def paulis_shape(dim: MatrixDimension) -> Tuple[int, ...]:
+    """Return just the shape of the paulis array for the given dimension.
+
+    Args:
+        dim: Dimension(s) of the Pauli matrices.
+
+    Returns:
+        Shape of the array obtained by ``paulis(dim)``.
+    """
+    if isinstance(dim, int):
+        dim = (dim,)
+
+    return tuple(np.square(dim)) + (tuple(np.prod(dim, keepdims=True)) * 2)
+
+
 def components(
     matrix: array_like,
     dim: Optional[MatrixDimension] = None,
     npmod: ModuleType = np
-) -> ndarray:
+) -> ArrayType:
     r"""Return the Pauli decomposition coefficients :math:`\nu_{k_1 \dots k_n}` of the matrix.
 
     Args:
@@ -245,7 +260,7 @@ def compose(
     components: array_like,
     dim: Optional[MatrixDimension] = None,
     npmod: ModuleType = np
-) -> ndarray:
+) -> ArrayType:
     r"""Compose a matrix from the Pauli components.
 
     Args:
@@ -298,7 +313,7 @@ def truncate(
     components: array_like,
     reduced_dim: MatrixDimension,
     npmod: ModuleType = np
-) -> ndarray:
+) -> ArrayType:
     r"""Truncate a component array of a matrix into the components for a submatrix.
 
     The component array can have extra dimensions in front (e.g. time axis if this is a time series of components).
@@ -364,28 +379,48 @@ def truncate(
     return components[(...,) + slices]
 
 
-def symmetry(dim: int):
+def symmetry(dim: MatrixDimension):
     r"""Return the symmetry (-1, 0, 1) of the Pauli matrices.
 
     Args:
         dim: Dimension of the Pauli matrices.
 
     Returns:
-        An 1D integer array with entries -1, 0, 1 depending on whether the corresponding Pauli matrix
+        An integer array with entries -1, 0, 1 depending on whether the corresponding Pauli matrix
         is antisymmetric, diagonal, or symmetric.
     """
+    if isinstance(dim, int):
+        dim = (dim,)
 
-    symmetry = np.zeros(dim ** 2, dtype=int)
+    subsystems = []
+
+    for pauli_dim in dim:
+        symmetry = np.zeros(pauli_dim ** 2, dtype=int)
+        ip = 1
+        for isub in range(1, pauli_dim):
+            for _ in range(isub):
+                symmetry[ip] = 1
+                ip += 1
+                symmetry[ip] = -1
+                ip += 1
 
-    ip = 1
-    for isub in range(1, dim):
-        for irow in range(isub):
-            symmetry[ip] = 1
-            ip += 1
-            symmetry[ip] = -1
             ip += 1
 
-        ip += 1
+        subsystems.append(symmetry)
+
+    # Compose symmetry combinations
+    # Truth table for two subsystems
+    #     -1  0  1
+    #   ----------
+    # -1|  1 -1 -1
+    #  0| -1  0  1
+    #  1| -1  1  1
+
+    symmetry = subsystems[0]
+    for subsystem in subsystems[1:]:
+        symprod = symmetry[..., None] * subsystem
+        symsum = symmetry[..., None] + subsystem
+        symmetry = symprod + np.where(symprod == 0, symsum, 0)
 
     return symmetry
 
